import com.declarativa.interprolog.SWISubprocessEngine;
//import com.declarativa.interprolog.swi.*;
//import com.declarativa.interprolog.*;
import java.io.*;
import javax.swing.*;
import javax.swing.table.*;
import java.awt.event.*;
import java.awt.*;


public class Sudoku extends JFrame{
	SWISubprocessEngine engine;
	SudokuModel model;
	JTable board;
	public Sudoku(SWISubprocessEngine en){
		engine=en;
		File f=new File("sudoku.pl");
		if(engine.consultAbsolute(f)){
			setJMenuBar(buildMenus());
			model = new SudokuModel();
			board = new SudokuBoard(model);
			
			getContentPane().add(BorderLayout.CENTER,board);
			this.pack();
			this.setVisible(true);
		}else{
			System.err.println("Brak pliku sudoku.pl");
			System.exit(0);
		}
		
	}
	JMenuBar buildMenus(){
		JMenuBar mb;
		mb=new JMenuBar();
		setJMenuBar(mb);
		JMenu fileMenu = new JMenu("File");
		JMenu puzzleMenu = new JMenu("Puzzle");
		mb.add(fileMenu);
		mb.add(puzzleMenu);
		JMenuItem clearTable = new JMenuItem("Clear");
		puzzleMenu.add(clearTable);
		clearTable.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				model.clean();
			}
		});
		JMenuItem solvePuzzle = new JMenuItem("Solve");
		puzzleMenu.add(solvePuzzle);
		solvePuzzle.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				String goal;
				goal="top([";
				for(int x=0;x<9;x++){
					for(int y=0;y<9;y++){
						Object val=model.getValueAt(x, y);
						if(val!=null){
							goal+=String.valueOf(val);
						}else{
							goal+="A"+String.valueOf(x+1)+String.valueOf(y+1);
						}
						goal+=",";
					}
				}
				goal=goal.substring(0, goal.length()-1);
				goal+="])";
				Object [] o=engine.deterministicGoal(goal, null);
				String res=o[0].toString();
				
				
				res=res.substring(5, res.length()-2);
				String [] res1=res.split(",");
				int count=0;
				
				for(int x=0;x<9;x++){
					for(int y=0;y<9;y++){
						model.setValueAtG(Integer.valueOf(res1[count]), x, y);
						count++;
					}
				}
				
			}
		});
		JMenuItem exitApp = new JMenuItem("Exit");
		fileMenu.add(exitApp);
		exitApp.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				engine.shutdown();
				System.exit(0);
			}
		});
		return mb;
	}
	static class SudokuBoard extends JTable{
		public SudokuBoard(SudokuModel sm){
			super(sm);
			setCellSelectionEnabled(true);
			setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
			TableColumnModel tcm = getColumnModel();
			Font font = getFont().deriveFont((float)20.0);
			setFont(font);
			int width = getFontMetrics(font).getMaxAdvance();
			setDefaultRenderer(Integer.class,new MyCellRenderer());	
			for(int c=0;c<tcm.getColumnCount();c++){
				((TableColumn)tcm.getColumn(c)).setMaxWidth(width);
			}
			setRowHeight(width);
		}
			public void paint(Graphics g){
				super.paint(g);
				g.setColor(Color.black);
				for(int r=3;r<9;r+=3){
					int Y = r*getRowHeight();
					g.drawLine(0,Y-1, getWidth()-1, Y-1);
					g.drawRect(0, Y-2, getWidth()-1,2);
					
				}
				for(int c=3;c<9;c+=3){
					int X = getCellRect(0,c,true).x;
					g.drawLine(X-1,0,X-1,getHeight()-1);
					g.drawRect(X-2, 0, 2, getHeight()-1);
				}
			}
			public TableCellEditor getDefaultEditor(Class columnClass){
				
				DefaultCellEditor T = (DefaultCellEditor)super.getDefaultEditor(columnClass);
				T.getComponent().setFont(getFont());
				return T;
			}
		
	}
	
	static class MyCellRenderer extends DefaultTableCellRenderer{
		public Component getTableCellRendererComponent(JTable table,Object value,boolean isSelected,boolean hasFocus,int row,int column){
			
			Component c = super.getTableCellRendererComponent(table,value,isSelected,hasFocus,row,column);
			
			if (table.getModel().isCellEditable(row,column)){
				c.setForeground(Color.black);
				//c.setBackground(Color.white);
			}
			else {
				c.setForeground(Color.blue);
				//c.setBackground(Color.lightGray);
			}
			((JLabel)c).setHorizontalAlignment(JLabel.CENTER);
			c.setFont(table.getFont());
			return c;
		}
	}
	
	static class SudokuModel extends AbstractTableModel implements Serializable{
		int [][] cells;
		boolean[][] given;
		SudokuModel(){
			cells=new int[9][9];
			given = new boolean[9][9];
			clean();
		}
		void clean(){
			for(int i=0;i<9;i++){
				for(int j=0;j<9;j++){
					cells[i][j]=0;
					given[i][j]=false;
				}
			}
			fireTableDataChanged();
		}
		public int getRowCount(){return cells.length;}
		public int getColumnCount(){return cells[0].length;}
		public Class getColumnClass(int columnIndex){return Integer.class;}
		public Object getValueAt(int x,int y){
			int value = cells[x][y];
			if(value==0)return null;
			else return new Integer(value);
		}
		public boolean getGiven(int x,int y){
			return given[x][y];
		}
		public void setGiven(int x,int y){
			given[x][y]=true;
		}
		public void setValueAt(Object aVal,int x, int y){
			int value = 0;
			if(aVal!=null) value = ((Integer)aVal).intValue();
			int oldValue = cells[x][y];
			if (value<0||value>getRowCount()) value=oldValue;
			cells[x][y] = value;
			given[x][y]=true;
			fireTableDataChanged();
		}
		public void setValueAtG(Object aVal,int x, int y){
			int value = 0;
			if(aVal!=null) value = ((Integer)aVal).intValue();
			int oldValue = cells[x][y];
			if (value<0||value>getRowCount()) value=oldValue;
			cells[x][y] = value;
			fireTableDataChanged();
			
		}
		public boolean isCellEditable(int x,int y){
			if(given[x][y])return false;
			else return true;
		}
	}
	
	public static void main(String[] args){
		
		final SWISubprocessEngine engine=new SWISubprocessEngine(args[0]);
		Sudoku s=new Sudoku(engine);
		s.addWindowListener(new WindowAdapter(){
			public void windowClosing(WindowEvent e){
				engine.shutdown();
				System.exit(0);
			}
		});
		
	}
}
